#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file stp_nw_image_generator.py
 @brief This module is generete netowork toplogy image file(png)
 @date $Date$

 @author 株式会社 東日本計算センター

 NewBSD

"""
import os.path
import sys
# import time
from datetime import datetime
# import RTC
# import STPPortIF
import matplotlib
matplotlib.rcParams['backend.qt4'] = 'PySide'
matplotlib.use('Qt4Agg')

import matplotlib.pyplot as plt
import networkx as nx
import random
import copy
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

B_COLOR = "limegreen"   # Base node color
SEL_COLOR = "deepskyblue"   # selected node color
R_COLOR = "orange"      # Robot node color
MISSING_COLOR = "red"   # missing node color

FONT_COLOR = "black"    # node name font color
FONT_SIZE = 18          # node name font size

B_SIZE = 3500           # Base node size
R_SIZE = 2500           # Robot node size

EDGE_WIDTH_B = 3        # blue edge width
EDGE_WIDTH_R = 2        # red edge width

CURRENT = os.getcwd()
IMG_PATH = CURRENT + "/STP_network_topology.png"

G = nx.Graph()


# h28-n
# NetworkTopology
class GenerateTopologyImage():

    def __init__(self):
        print "name of the current matplotlib backend =" +\
            matplotlib.get_backend()

        self._init_variable()

    def _init_variable(self):
        self._node_list = []
        self._base_node_id = ""
        self._edge_list = []
        self._get_sel_node_func = None
        self._get_route_list_func = None
        self._get_missing_node_list_func = None
        self._get_base_node_id_func = None
        self._get_pos_change_func = None
        self._pos_change = False
        self._gpos = ""
        self._sel_id = ""

    def generate_image(self):
        route_list = sorted(self._get_route_list_func(),
                            key=lambda tim: tim.tm.sec)

        missing_list = self._get_missing_node_list_func()
        self._sel_id = self._get_sel_node_func()
        self._base_node_id = self._get_base_node_id_func()
        self._pos_change = self._get_pos_change_func()

        self._node_list_create(route_list)
        self._node_edge_list_create(route_list, missing_list)

        self._topology_image_draw(missing_list)

    def _node_list_create(self, r_list):
        del self._node_list[:]
        for r in r_list:
            self._node_list.append(r.id)

    def _node_edge_list_create(self, r_list, m_list):
        del self._edge_list[:]
        list_temp = []

        for r in r_list:
            idx = r.id
            for v in r.data:
                conx = v.Connected_Node

                if not idx or not conx or not v.PortState:
                    pass
                elif idx in m_list or conx in m_list:
                    pass
                elif v.PortState == "disabled":
                    pass
                elif conx in self._node_list:
                    if v.PortState == "forwarding":
                        colx = "blue"
                        widthx = EDGE_WIDTH_B
                    else:
                        colx = "red"
                        widthx = EDGE_WIDTH_R

                    temp_edge_list = (idx, conx, colx, widthx)
                    list_temp.append(temp_edge_list)
                else:
                    pass

        temp = self._edge_list_check(list_temp)
        self._edge_list = copy.deepcopy(temp)

    def _edge_list_check(self, list):
        retval = []
        for nodx1, nodx2, colx, wthx in list:
            for nody1, nody2, coly, wthy in list:
                if nodx1 == nody2 and nodx2 == nody1:
                    if colx == coly:
                        temp = (nodx1, nodx2, colx, wthx)
                    else:
                        temp = (nodx1, nodx2, "red", EDGE_WIDTH_R)
                    retval.append(temp)
                    break
        return retval

    def _topology_image_draw(self, missing_list):
        n_sizes = []
        n_colors = []

        self._node_add_graph(self._node_list, missing_list)

        if self._edge_list:
            for x, y, r, w in self._edge_list:
                G.add_edge(x, y, color=r, width=w)

            edges = G.edges()
            e_colors = [G[u][v]['color'] for u, v in edges]
            e_linewidth = [G[u][v]['width'] for u, v in edges]
        else:
            e_colors = "black"
            e_linewidth = 1

        self._node_attr_set(n_colors, n_sizes)
        if self._pos_change:
            self._gpos = nx.spring_layout(G)
            self._pos_change = False
        nx.draw(G, pos=self._gpos, with_labels=True, font_size=FONT_SIZE,
                font_color=FONT_COLOR, edge_color=e_colors, width=e_linewidth,
                node_size=n_sizes, node_color=n_colors)

        plt.savefig(IMG_PATH)                    # Save the current figure.
        # clean the old graph in pyplot.
        plt.clf()
        # Remove all nodes and edges from the graph.
        G.clear()

    def _node_attr_set(self, colors, sizes):
        nodes = G.nodes(data=True)

        for i, attr in nodes:
            colors.append(attr["node_color"])
            sizes.append(attr["node_size"])

    def _node_add_graph(self, list, miss_list):
        for i in list:
            siz = R_SIZE
            col = R_COLOR
            if i == self._base_node_id[0]:
                col = B_COLOR
                siz = B_SIZE
            if i == self._sel_id:
                col = SEL_COLOR
            for u in miss_list:
                if u == i:
                    col = MISSING_COLOR

            G.add_node(i, node_size=siz, node_color=col)

    def set_sel_node_func(self, func):
        self._get_sel_node_func = func

    def set_route_list_func(self, func):
        self._get_route_list_func = func

    def set_missing_node_list_func(self, func):
        self._get_missing_node_list_func = func

    def set_base_node_id_func(self, func):
        self._get_base_node_id_func = func

    def set_pos_change_func(self, func):
        self._get_pos_change_func = func


def main():
    pass


if __name__ == "__main__":
    main()
